const process = require('process')
const { exec } = require('child_process')
const util = require('util')
const { existsSync } = require('fs')
const chalk = require('chalk')
const ora = require('ora')
const boxen = require('boxen')

const execAsync = util.promisify(exec)

const copyRepository = function({ sourceUser, sourceRepo, targetRepo, targetDesc, targetVisi, changeAuthor }) {
  let spinner
  (async () => {
    // Get repository list to copy
    spinner = ora("Getting information from source user account\n").start()
    
    let copyRepoList = []
    const sourceRepoListStr = await execAsync(`gh repo list ${sourceUser}`)
    const sourceRepoList = sourceRepoListStr.stdout.split('\n')
    sourceRepoList.forEach((repoStr) => {
      const info = repoStr.split('\t')
      const sourceRepoName = info[0].split('/')[1]
      if (!sourceRepo){
        copyRepoList.push({
          sourceRepo: sourceRepoName,
          targetRepo: sourceRepoName,
          targetDesc: info[1],
          targetVisi: targetVisi ? targetVisi : info[2],
        })
      } else if (sourceRepo == sourceRepoName) {
        copyRepoList.push({
          sourceRepo: sourceRepo,
          targetRepo: targetRepo ? targetRepo: sourceRepo,
          targetDesc: targetDesc ? targetDesc : info[1],
          targetVisi: targetVisi ? targetVisi : info[2],
        })
      }
    })

    spinner.succeed()

    // make my-repository folder to perform operations
    const repositoryFolder = "my-repository"

      // Remove folder in case if it already exists.
    if (existsSync(`./${repositoryFolder}`)) {
      await execAsync(
        `${
          process.platform === "win32" ? "rmdir /s /q" : "rm -rf"
        } ${repositoryFolder}`
      )
    }

      // Create folder.
    await execAsync(`mkdir ${repositoryFolder}`)
    process.chdir(repositoryFolder)

    // copy repositories
    const usernameOut = await execAsync(`git config user.name`)
    const emailOut = await execAsync(`git config user.email`)
    const username = usernameOut.stdout.replace('\n', '')
    const email = emailOut.stdout.replace('\n', '')

    spinner = ora("Copying repository\n").start()

    for (let i = 0; i < copyRepoList.length; ++i) {
      const repo = copyRepoList[i]
      
      // clone source repository
      spinner.text = `Copying repository ${repo.sourceRepo} - Cloning repository ${sourceUser}/${repo.sourceRepo}\n`
      await execAsync(`git clone --bare https://github.com/${sourceUser}/${repo.sourceRepo}.git`)

      process.chdir(`${repo.sourceRepo}.git`)

      // change author of commits
      spinner.text = `Copying repository ${repo.sourceRepo} - Changing author(${sourceUser} -> ${username})\n`
      if (changeAuthor){
        await execAsync(`git filter-branch --commit-filter 'if [ $GIT_AUTHOR_NAME = "${sourceUser}" ]; then export GIT_AUTHOR_NAME="${username}";
          export GIT_AUTHOR_EMAIL=${email}; export GIT_COMMITTER_NAME="${username}"; export GIT_COMMITTER_EMAIL=${email}; fi; 
          git commit-tree "$@"'`)
      } else {
        await execAsync(`git filter-branch --commit-filter 'export GIT_AUTHOR_NAME="${username}";
          export GIT_AUTHOR_EMAIL=${email}; export GIT_COMMITTER_NAME="${username}"; export GIT_COMMITTER_EMAIL=${email};
          git commit-tree "$@"'`)
      }

      // create target repository
      spinner.text = `Copying repository ${repo.sourceRepo} - Creating repository ${username}/${repo.targetRepo}\n`
      await execAsync(`gh repo create ${repo.targetRepo} -d "${repo.targetDesc}" --${repo.targetVisi}`)

      // push commits
      spinner.text = `Copying repository ${repo.sourceRepo} - Pushing commits ${username}/${repo.targetRepo}\n`
      await execAsync(`git remote set-url origin git@github.com:${username}/${repo.targetRepo}.git`)
      await execAsync(`git push --mirror -u`)

      process.chdir('..')
    }

    spinner.succeed()

    // remove my-repository folder
    spinner = ora("Removing cache\n").start()
    
    process.chdir('..')
    await execAsync(
      `${
        process.platform === "win32" ? "rmdir /s /q" : "rm -rf"
      } ${repositoryFolder}`
    )

    spinner.succeed()

    console.log(
      boxen(
        `${chalk.green("Success")} ${
          copyRepoList.length
        } repositories have been created.
      If you rely on this tool, please consider buying me a coffee to send a little ETH. I would appreciate it.
      My address is 0x135c823878da9F9d71e9Fc486fA2FE62cE9861a3`,
        { borderColor: "yellow", padding: 1, align: "center" }
      )
    )
  }) ()
}

module.exports = copyRepository