const yargs = require('yargs')

const { COMMAND } = require('../_constants/index.js')

const argv = yargs
  .usage('node $0 <command> [options]')
  .command(COMMAND.history, 'Generate your GitHub activity history', yargs => {
    return yargs
      .option({
        workdaysOnly: {
          alias: 'w',
          description: 'If set, generate activity on work days only',
          type: 'string',
        },
        commitsPerDay: {
          alias: 'c',
          description: 'Customize how many commits a day to make. Default is "0,3"',
          type: 'string',
        },
        startDate: {
          alias: 's',
          description: 'Start date in yyyy-MM-dd format. Default is the last year',
          type: 'string',
        },
        endDate: {
          alias: 'e',
          description: 'End date in yyyy-MM-dd format. Default is now',
          type: 'string',
        },
        repositoryName: {
          alias: 'r',
          description: 'New repository name to contain activity history. Default is "history-<startDate>-<endDate>"',
          type: 'string'
        }
      })
      .check((arg, options) => {
        return true
      })
  })
  .command(COMMAND.repository, `Copy other's repository into yours with full commit history and authored by you`, yargs => {
    return yargs
      .option({
        sourceUser: {
          alias: 'u',
          description: 'Source username to copy repository',
          type: 'string'
        }
      })
      .option({
        sourceRepo: {
          alias: 's',
          description: 'Name of repository to copy. If not set, all repositories of the user is copied',
          type: 'string'
        }
      })
      .option({
        targetRepo: {
          alias: 't',
          description: 'When sourceRepo is set, name of new repository in your GitHub account. Default is sourceRepo',
          type: 'string'
        }
      })
      .option({
        targetDesc: {
          alias: 'd',
          description: "When sourceRepo is set, description of new repository in your GitHub account. Default is sourceRepo's description",
          type: 'string'
        }
      })
      .option({
        targetVisi: {
          alias: 'b',
          description: `Visibility of new repositories in your GitHub account. Default is to keep sourceRepo's visibility`,
          type: 'string'
        }
      })
      .option({
        changeAuthor: {
          alias: 'a',
          description: `This is previous author name to be changed. Our goal is to make you author the commits. If not set, all commits are authored by you`,
          type: 'string'
        }
      })
      .check((arg, options) => {
        if (!('sourceUser' in arg) || !arg.sourceUser){
          throw new Error("Enter source username to copy repository")
        }

        return true
      })
  })
  .check((arg, options) => {
    arg.command = arg._[0]
    if (!(arg._[0] in COMMAND)) {
      throw new Error('Enter valid command')
    }
    return true
  })
  .help()
  .alias('version', 'v')
  .alias('help', 'h').argv

module.exports = argv